#include <wx/wx.h>
#include <wx/msgdlg.h>
#include <wx/regex.h>
#include <wx/aboutdlg.h>
#include "DialogOptions.h"
#include "ReservationEdition.h"
#include "MyFrame.h"

// turn debug mode on (1) to enter debug mode
#define DEBUGMODE 0


int idMenuRead = ::wxNewId();
int idMenuWrite = ::wxNewId();
int idMenuOptions = ::wxNewId();
int idMenuQuit = ::wxNewId();
int idMenuAdd = ::wxNewId();
int idMenuModify = ::wxNewId();
int idMenuDelete = ::wxNewId();
int idMenuAbout = ::wxNewId();
int idBtnRead = ::wxNewId();
int idBtnWrite = ::wxNewId();
int idBtnAdd = ::wxNewId();
int idBtnModify = ::wxNewId();
int idBtnDelete = ::wxNewId();
int idLstCtrlReservations = ::wxNewId();


BEGIN_EVENT_TABLE(MyFrame, wxFrame)
	EVT_MENU(idMenuRead, MyFrame::OnMenuRead)
	EVT_MENU(idMenuWrite, MyFrame::OnMenuWrite)
	EVT_MENU(idMenuOptions, MyFrame::OnMenuOptions)
	EVT_MENU(idMenuQuit, MyFrame::OnMenuQuit)

	EVT_MENU(idMenuAdd, MyFrame::OnMenuAdd)
	EVT_MENU(idMenuModify, MyFrame::OnMenuModify)
	EVT_MENU(idMenuDelete, MyFrame::OnMenuDelete)
	EVT_MENU(idMenuAbout, MyFrame::OnMenuAbout)

	EVT_BUTTON(idBtnRead, MyFrame::OnClickRead)
	EVT_BUTTON(idBtnWrite, MyFrame::OnClickWrite)
	EVT_BUTTON(idBtnAdd, MyFrame::OnClickAdd)
	EVT_BUTTON(idBtnModify, MyFrame::OnClickModify)
	EVT_BUTTON(idBtnDelete, MyFrame::OnClickDelete)

    EVT_LIST_ITEM_ACTIVATED(idLstCtrlReservations, MyFrame::OnDblClickItem)

    EVT_CLOSE(MyFrame::OnCloseWindow)
END_EVENT_TABLE()


//*** constructors / destructors

MyFrame::MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size)
: wxFrame((wxFrame*)NULL, -1, title, pos, size)
{
    this->SetIcon(wxICON(APPICON));

    //* drawing of menu bar

	wxMenuBar* menuBar = new wxMenuBar();
	wxMenu* menuFile = new wxMenu();
	wxMenu* menuEdit = new wxMenu();
	wxMenu* menuHelp = new wxMenu();

    menuFile->Append(idMenuRead, "Read from registry", "Read DHCP reservations from registry");
    menuFile->Append(idMenuWrite, "Write into registry", "Write DHCP reservations into registry");
	menuFile->AppendSeparator();
    menuFile->Append(idMenuOptions, "Options", "Options");
	menuFile->AppendSeparator();
	menuFile->Append(idMenuQuit, "Quit", "Quit application");
	menuBar->Append(menuFile, "File");

	menuEdit->Append(idMenuAdd, "Add", "Add DHCP reservation");
	menuEdit->Append(idMenuModify, "Modify", "Modify DHCP reservation");
	menuEdit->Append(idMenuDelete, "Delete", "Delete DHCP reservation");
	menuBar->Append(menuEdit, "Edit");

	menuHelp->Append(idMenuAbout, "About", "Information about application");
	menuBar->Append(menuHelp, "Help");

	this->SetMenuBar(menuBar);

	this->CreateStatusBar();
	this->SetStatusText(wxEmptyString);

	//* drawing of GUI

    wxPanel* panelMain = new wxPanel(this, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
	wxBoxSizer* sizerMain = new wxBoxSizer(wxHORIZONTAL);

	panelMain->SetSizer(sizerMain);
    sizerMain->AddSpacer(5);

        wxPanel* panelButtons = new wxPanel(panelMain, -1, wxDefaultPosition, wxDefaultSize, wxTAB_TRAVERSAL);
        sizerMain->Add(panelButtons, 0, wxEXPAND);

        wxBoxSizer* sizerButtons = new wxBoxSizer(wxVERTICAL);
        panelButtons->SetSizer(sizerButtons);

            sizerButtons->AddSpacer(20);
            this->btnRead = new wxButton(panelButtons, idBtnRead, "Read", wxDefaultPosition, wxSize(80,30));
            sizerButtons->Add(this->btnRead, 0, wxEXPAND);
            sizerButtons->AddSpacer(5);
            this->btnWrite = new wxButton(panelButtons, idBtnWrite, "Write", wxDefaultPosition, wxSize(80,30));
            sizerButtons->Add(this->btnWrite, 0, wxEXPAND);
            sizerButtons->AddSpacer(20);
            this->btnAdd = new wxButton(panelButtons, idBtnAdd, "Add", wxDefaultPosition, wxSize(80,20));
            sizerButtons->Add(this->btnAdd, 0, wxEXPAND);
            sizerButtons->AddSpacer(5);
            this->btnModify = new wxButton(panelButtons, idBtnModify, "Modify", wxDefaultPosition, wxSize(80,20));
            sizerButtons->Add(this->btnModify, 0, wxEXPAND);
            sizerButtons->AddSpacer(5);
            this->btnDelete = new wxButton(panelButtons, idBtnDelete, "Delete", wxDefaultPosition, wxSize(80,20));
            sizerButtons->Add(this->btnDelete, 0, wxEXPAND);
            sizerButtons->AddSpacer(5);

    sizerMain->AddSpacer(5);

        this->lstctrlReservations = new ArqWx::ListCtrl(panelMain, idLstCtrlReservations, wxDefaultPosition, wxDefaultSize, wxLC_REPORT | wxLC_SINGLE_SEL);
        sizerMain->Add(this->lstctrlReservations, 1, wxEXPAND);

        this->lstctrlReservationsBackUp = new ArqWx::ListCtrl();
        this->lstctrlReservationsBackUp->Create(panelMain, -1, wxDefaultPosition, wxDefaultSize, wxLC_REPORT | wxLC_SINGLE_SEL);
        sizerMain->Add(this->lstctrlReservationsBackUp, 1, wxEXPAND);
        #if DEBUGMODE == 0
        this->lstctrlReservationsBackUp->Hide();
        #else
        this->SetSize(710,280);
        sizerMain->Layout();
        #endif
        this->lstctrlReservationsBackUp->AddColumn("", wxLIST_FORMAT_LEFT, 130);
        this->lstctrlReservationsBackUp->AddColumn("", wxLIST_FORMAT_LEFT, 110);
        this->lstctrlReservationsBackUp->AddColumn("", wxLIST_FORMAT_LEFT, 40);

    this->lstctrlReservations->SetRowBackgroundColourAlternation(true);

	this->lstctrlReservations->AddColumn("MAC Address", wxLIST_FORMAT_LEFT, 130);
	this->lstctrlReservations->AddColumn("IP Address", wxLIST_FORMAT_LEFT, 110);
	this->lstctrlReservations->AddColumn("Flag", wxLIST_FORMAT_LEFT, 40);

    //* initializations

    this->rkeyApp = new wxRegKey("HKEY_LOCAL_MACHINE\\Software\\TFTPD32\\DHCP");

	this->ReadReservations();
}

MyFrame::~MyFrame()
{
    delete this->rkeyApp;
}


//*** specific methods

//* bool IsModified()
//
// Checks if any reservation has been changed
//
// parameters:
// - <none>
// returns:
// - (bool): flag which noticies if DHCP reservations list has been modified since last reading from registry

bool MyFrame::IsModified()
{
    bool change = false; // change flag

    for ( int i = 0 ; (i < this->lstctrlReservations->GetItemCount()) && !change ; i++ ) { // reads each item in the list
        if ( !(this->lstctrlReservations->GetString(i, 2)).IsEmpty() ) { // if flag value is not empty (which means 'A'/'M'/'D'/'MD')
            change = true; // sets change flag to true
        }
    }

    return change; // returns change flag value
}


//* void ReadReservations()
//
// Reads reservations from registry
//
// parameters:
// - <none>
// returns:
// - <none>

void MyFrame::ReadReservations()
{
    bool reloadAfterModification = true; // reload flag

    //* confirmation dialog

    if ( this->IsModified() ) { // if DHCP reservations list has been changed
        reloadAfterModification = false;

        int returnCode = ::wxMessageBox("Are you sure you want to reload DHCP reservations\nfrom registry and discard modifications?", "Changes made", wxYES_NO | wxICON_EXCLAMATION | wxNO_DEFAULT);

        if ( returnCode == wxYES ) {
            reloadAfterModification = true;
        }
    }

    //* reading

    if ( reloadAfterModification ) { // if reload flag is set to true
        long li = 0; // dummy variable
        wxString valueName1, valueName2; // regkey values name (2 to detect end of values list)
        wxString strValue; // regkey value
        wxRegEx regxMACAddr("^(([[:xdigit:]]{2}):){5}([[:xdigit:]]{2})$"); // regular expression for MAC address (6 hexadecimal digits separated by colons)
        wxRegEx regxIPAddr("^(([0-9]{1}|[0-9]{2}|[0-1][0-9]{2}|2[0-4][0-9]|25[0-5])\\.){3}([0-9]{1}|[0-9]{2}|[0-1][0-9]{2}|2[0-4][0-9]|25[0-5])$"); // regular expression for IP address (4 [0-255] numbers separated by periods)

        this->lstctrlReservations->DeleteAllItems(); // cleans DHCP reservations list
        this->lstctrlReservationsBackUp->DeleteAllItems(); // cleans DHCP reservations list

        if( !this->rkeyApp->Exists() ) { // if application regkey does not exist
            this->rkeyApp->Create(); // creates application regkey
        }

        this->rkeyApp->GetFirstValue(valueName1, li); // gets first value name

        do { // reads application regkey values names...
            if ( regxMACAddr.Matches(valueName1) ) { // if value name matches a MAC address
                this->rkeyApp->QueryValue(valueName1, strValue); // retrieves matching IP address

                if ( regxIPAddr.Matches(strValue) ) { // if value data matches a IP address
                    wxString item[3] = { // new item
                        valueName1, // read MAC address
                        strValue, // read IP address
                        wxEmptyString };

                    this->lstctrlReservations->AddItemRow(item); // adds item to DHCP reservations list
                    this->lstctrlReservationsBackUp->AddItemRow(item); // adds item to backup DHCP reservations list
                }
            }

            valueName2 = valueName1; // backups value name (useful to detect the end of the list)

            this->rkeyApp->GetNextValue(valueName1, li); // gets next value
        }
        while ( valueName1 != valueName2 ); // ...until current value name equals previous one (only way to detect the end of the MAC addresses list)
    }
}

//* void MyFrame::WriteReservations()
//
// Writes reservations into registry
//
// parameters:
// - <none>
// returns:
// - <none>

void MyFrame::WriteReservations()
{
    bool writeDHCPReservations = false; // writing flag

    //* confirmation dialog

    if ( this->IsModified() ) { // if DHCP reservations list has been changed
        int returnCode = ::wxMessageBox("Are you sure you want to write\nmodifications into registry?", "Write confirmation", wxYES_NO | wxICON_QUESTION | wxNO_DEFAULT);

        if ( returnCode == wxYES ) {
            writeDHCPReservations = true;
        }
    }

    //* writing

    if ( writeDHCPReservations ) { // if writing flag is set to true
        for ( int i = 0 ; i < this->lstctrlReservations->GetItemCount() ; i++ ) { // reads each item in the DHCP reservations list
            wxString MACAddr, IPAddr, flag; // current item' MAC address, IP address and flag

            MACAddr = this->lstctrlReservations->GetString(i, 0); // stores MAC address value
            IPAddr = this->lstctrlReservations->GetString(i, 1); // stores IP address value
            flag = this->lstctrlReservations->GetString(i, 2); // stores flag value

            if ( !flag.IsEmpty() ) { // if flag value is not empty (which means 'A'/'M'/'D'/'MD')
                if ( flag.IsSameAs("A") ) { // if flag value equals 'A' (Added item)
                    this->rkeyApp->SetValue(MACAddr, IPAddr); // modifies regkey value
                }

                if ( flag.IsSameAs("M") ) { // if flag value equals 'M' (Modified item)
                    if ( !MACAddr.IsSameAs(this->lstctrlReservationsBackUp->GetString(i)) ) { // if reservation MAC address has been changed
                        this->rkeyApp->RenameValue(this->lstctrlReservationsBackUp->GetString(i), MACAddr); // renames regkey value
                    }
                    this->rkeyApp->SetValue(MACAddr, IPAddr); // modifies regkey value
                }

                if ( flag.EndsWith("D") ) { // if flag value ends with 'D' (Deleted item)
                    this->rkeyApp->DeleteValue(MACAddr); // deletes regkey value
                }

                this->lstctrlReservations->SetString(i, 2, wxEmptyString); // resets flag to <none>
            }

        }

        this->ReadReservations(); // reload DHCP reservations list from registry
    }
}


//* Reservation GetReservationFromItem(long reservationItemIdx)
//
// Gets a reservation object upon given listcontrol item index
//
// parameters:
// - reservationItemIdx (long): reservation item index
// returns:
// - (Reservation): reservation

Reservation MyFrame::GetReservationFromItem(long reservationItemIdx)
{
    Reservation tempReservation(
        this->lstctrlReservations->GetString(reservationItemIdx, 0),
        this->lstctrlReservations->GetString(reservationItemIdx, 1));

    return tempReservation;
}

//* void SetItemFromReservation(const Reservation& reservation, long reservationItemIdx)
//
// Sets given listcontrol item upon given reservation object
//
// parameters:
// - reservation (const Reservation&): reservation reference
// - reservationItemIdx (long): reservation item index
// returns:
// - <none>

void MyFrame::SetItemFromReservation(const Reservation& reservation, long reservationItemIdx)
{
    this->lstctrlReservations->SetString(reservationItemIdx, 0, reservation.GetMACAddress());
    this->lstctrlReservations->SetString(reservationItemIdx, 1, reservation.GetIPAddress());
}

//* long AddReservationItem(const Reservation& reservation)
//
// Adds a new item upon given reservation
//
// parameters:
// - reservation (const Reservation&): reservation reference
// returns:
// - (long): 0-based index of inserted or modified reservation item

long MyFrame::AddReservationItem(const Reservation& reservation)
{
    long newReservationItemIdx = this->lstctrlReservations->GetItemCount(); // gets current reservation items count
    this->lstctrlReservations->InsertItem(newReservationItemIdx, wxEmptyString); // inserts a new reservation item

    this->SetItemFromReservation(reservation, newReservationItemIdx); // inserts new reservation object into reservations listcontrol
    this->lstctrlReservations->SetString(newReservationItemIdx, 2, "A");

    return newReservationItemIdx;
}

//* long AddReservation()
//
// Adds a new reservation to reservations listcontrol
//
// parameters:
// - <none>
// returns:
// - (long): 0-based index of new reservation item

long MyFrame::AddReservation()
{
    long newReservationItemIdx = -1;

    wxString defaultMACAddress = "00:00:00:00:00:00";
    wxString defaultIPAddress = "0.0.0.0";

    if ( this->lstctrlReservations->GetItemCount() > 0 ) {
        defaultMACAddress = wxEmptyString;
        defaultIPAddress = wxEmptyString;
    }

    Reservation newReservation(
        defaultMACAddress, // default MAC address
        defaultIPAddress); // default IP address

    if ( this->EditReservation(newReservation) ) {
        newReservationItemIdx = this->AddReservationItem(newReservation);
    }

    return newReservationItemIdx;
}

//* void ModifyReservation(long currentReservationItemIdx)
//
// Allows to modify a reservation listcontrol item
//
// parameters:
// - currentReservationItemIdx (long): reservation index to modify
// returns:
// - <none>

void MyFrame::ModifyReservation(long currentReservationItemIdx)
{
    bool undeleted = true; // undeleted item flag
    wxString flag;
    wxString strFlag; // temporary string to store item flag once 'D'-trimmed

    flag = this->lstctrlReservations->GetString(currentReservationItemIdx, 2); // backups flag value

    //* management if it is a deleted item
    if ( flag.EndsWith("D", &strFlag) ) { // if current flag value ends with 'D' ('D'/'MD'), and stores rest of the string
        undeleted = false;

        int returnCode = ::wxMessageBox("Do you want to undelete this\nDHCP reservation?", "DHCP Reservation deleted", wxYES_NO | wxICON_EXCLAMATION | wxNO_DEFAULT);

        if ( returnCode == wxYES ) {
            this->lstctrlReservations->SetItem(currentReservationItemIdx, 2, strFlag); // sets item flag value to rest (trimmed by 'D')
            undeleted = true;
        }
    }

    //* edition of DHCP reservation
    if ( undeleted ) { // if the item was not previously undeleted
        Reservation tempReservation;

        tempReservation = this->GetReservationFromItem(currentReservationItemIdx); // sets temporary reservation object to current reservation item

        if ( this->EditReservation(tempReservation) ) {
            this->SetItemFromReservation(tempReservation, currentReservationItemIdx);
            this->lstctrlReservations->SetString(currentReservationItemIdx, 2, "M"); // sets item flag as modified
        }
    }
}

//* void DeleteReservation()
//
// Deletes a reservation
//
// parameters:
// - <none>
// returns:
// - <none>

void MyFrame::DeleteReservation()
{
    long currentItemIdx; // selected item index

    currentItemIdx = this->lstctrlReservations->GetSelectedItemIndex(); // gets selected item index
    if ( currentItemIdx != -1 ) { // if an item is selected
        wxString flag = this->lstctrlReservations->GetString(currentItemIdx, 2); // gets flag value

        if ( flag.IsSameAs("A") ) { // if flag value is 'A' (this is an Added item, so it has not been written into registry yet)
            this->lstctrlReservations->DeleteItem(currentItemIdx); // deletes the item from the list
        }
        else { // if flag value is not 'A' (the item has been written in the registry yet)
            if( !flag.EndsWith("D") ) { // if flag value is not already 'D'
                this->lstctrlReservations->SetString(currentItemIdx, 2, flag + "D"); // adds 'D' to item flag value (so it may be 'D' or 'MD' if the item has been previously Modified)
            }
        }
    }
}

//* bool EditReservation(Reservation& reservation)
//
// Edits given reservation
//
// parameters:
// - reservation (Reservation&): reservation to edit
// returns:
// - (bool): edition validation flag

bool MyFrame::EditReservation(Reservation& reservation)
{
    bool rtn = false;
    wxArrayString macAddresses;
    wxArrayString ipAddresses;
    ReservationEdition* dlgDHCPReservation; // dialog to edit DHCP reservation values

    // building of existing MAC & IP addresses lists
    for ( long itemIdx = 0 ; itemIdx < this->lstctrlReservations->GetItemCount() ; itemIdx++ ) {
        if ( this->lstctrlReservations->GetString(itemIdx, 0) != reservation.GetMACAddress() ) {
            macAddresses.Add(this->lstctrlReservations->GetString(itemIdx, 0));
        }
        if ( this->lstctrlReservations->GetString(itemIdx, 1) != reservation.GetIPAddress() ) {
            ipAddresses.Add(this->lstctrlReservations->GetString(itemIdx, 1));
        }
    }

    // MAC addresses edition
    dlgDHCPReservation = new ReservationEdition(this, -1, "DHCP Reservation", this->GetPosition() + wxSize(10,10), reservation, macAddresses, ipAddresses); // creates the dialog with current MAC address and IP address values
    dlgDHCPReservation->ShowModal();

    rtn = ( dlgDHCPReservation->GetReturnCode() == wxID_OK ); // shows dialog

    delete dlgDHCPReservation;

    return rtn;
}


//*** menu events

void MyFrame::OnMenuRead(wxCommandEvent& WXUNUSED(event))
{
    this->ReadReservations(); // launches DHCP reservations reading from registry
}

void MyFrame::OnMenuWrite(wxCommandEvent& WXUNUSED(event))
{
    this->WriteReservations(); // launches DHCP reservations writing into registry
}

void MyFrame::OnMenuOptions(wxCommandEvent& WXUNUSED(event))
{
    wxString rkApp; // application regkey path (as string, and as pointer to allow dialog to directly modify its value)
    DialogOptions* dlgOptions; // dialog to edit options

    rkApp = this->rkeyApp->GetName(); // gets application regkey path value

    dlgOptions = new DialogOptions(this, -1, "Options", this->GetPosition() + wxSize(20,20), rkApp); // creates dialog to edit options

    if ( dlgOptions->ShowModal() == wxID_OK ) { // shows dialog and if return code is 'ok'
        this->rkeyApp->SetName(rkApp); // sets application regkey path value
    }

    dlgOptions->Destroy();
}

void MyFrame::OnMenuQuit(wxCommandEvent& WXUNUSED(event))
{
    this->Close(TRUE); // closes the frame (implicitely generating CLOSE_EVENT)
}

void MyFrame::OnMenuAdd(wxCommandEvent& event)
{
    this->OnClickAdd(event); // launches DHCP reservation addition
}

void MyFrame::OnMenuModify(wxCommandEvent& event)
{
    this->OnClickModify(event); // launches DHCP reservation modification
}

void MyFrame::OnMenuDelete(wxCommandEvent& event)
{
    this->OnClickDelete(event); // launches DHCP reservation deletion
}

void MyFrame::OnMenuAbout(wxCommandEvent& WXUNUSED(event))
{
	wxAboutDialogInfo aboutDlgInfo;
	aboutDlgInfo.SetName("Tftpd32-DHCP");
	aboutDlgInfo.SetVersion("v1.3");
	aboutDlgInfo.SetCopyright("(2010/03)");
	aboutDlgInfo.SetDescription("DHCP manager for Tftpd32.\n\nby TrV\ncontact: dev(at)arqendra(dot)net\n\nDeveloped with:\n- Code::Blocks 8.02 (nightly build #6181)\n- MinGW 3.4.5\n- wxWidgets 2.8.7\n- NConvert 4.95\n- UPX 3.04w\n");
	aboutDlgInfo.SetWebSite("http://dev.arqendra.net");

	::wxAboutBox(aboutDlgInfo);
}


//*** clic events

void MyFrame::OnClickRead(wxCommandEvent& WXUNUSED(event))
{
    this->ReadReservations(); // launches DHCP reservations reading from registry
}

void MyFrame::OnClickWrite(wxCommandEvent& WXUNUSED(event))
{
    this->WriteReservations(); // launches DHCP reservations writing into registry
}


void MyFrame::OnClickAdd(wxCommandEvent& WXUNUSED(event))
{
    this->AddReservation();
}

void MyFrame::OnClickModify(wxCommandEvent& WXUNUSED(event))
{
    long currentItemIdx; // selected item index

    currentItemIdx = this->lstctrlReservations->GetSelectedItemIndex(); // gets selected item index
    if ( currentItemIdx != -1 ) { // if an item is selected
        this->ModifyReservation(currentItemIdx); // launches DHCP reservation modification
    }
}

void MyFrame::OnClickDelete(wxCommandEvent& WXUNUSED(event))
{
    this->DeleteReservation(); // launches DHCP reservation deletion
}


//*** frame events

void MyFrame::OnCloseWindow(wxCloseEvent& WXUNUSED(event))
{
    bool quit = true; // quit flag

    //* confirmation dialog
    if ( this->IsModified() ) { // if any change has been made to DHCP reservations list
        quit = false;

        int returnCode = ::wxMessageBox("Are you sure you want to quit\nand discard modifications?", "Changes made", wxYES_NO | wxICON_EXCLAMATION | wxNO_DEFAULT);

        if ( returnCode == wxYES ) {
            quit = true;
        }
    }

    //* application exit
	if ( quit ) { // if quit flag is set to true
	    this->Destroy(); // destroys the frame, thus the application
	}
}


//*** listctrl events

void MyFrame::OnDblClickItem(wxListEvent& event)
{
    this->ModifyReservation(event.m_itemIndex); // launches DHCP reservation modification for item which has been double-clicked (/selected)
}
