#ifndef MYFRAME_H
#define MYFRAME_H

#include <wx/wx.h>
#include <wx/msw/registry.h>
#include "ArqWx.ListCtrl.h"
#include "Reservation.h"


//** class MyFrame
//
// main frame
//
// derived from: wxFrame

class MyFrame : public wxFrame
{
	public:
        // constructor(title, position, size)
        MyFrame(const wxString& title, const wxPoint& pos, const wxSize& size);
        // destructor
		~MyFrame();

        void OnMenuRead(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Read' menu item is selected
        void OnMenuWrite(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Write' menu item is selected
        void OnMenuOptions(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Read' menu item is selected
        void OnMenuQuit(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Options' menu item is selected

        void OnMenuAdd(wxCommandEvent& event); // actions executed when 'Add' menu item is selected
		void OnMenuModify(wxCommandEvent& event); // actions executed when 'Modify' menu item is selected
		void OnMenuDelete(wxCommandEvent& event); // actions executed when 'Delete' menu item is selected

		void OnMenuAbout(wxCommandEvent& WXUNUSED(event)); // actions executed when 'About' menu item is selected

		void OnClickRead(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Read' button is clicked
		void OnClickWrite(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Write' button is clicked

		void OnClickAdd(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Add' button is clicked
		void OnClickModify(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Modify' button is clicked
		void OnClickDelete(wxCommandEvent& WXUNUSED(event)); // actions executed when 'Delete' button is clicked

        void OnCloseWindow(wxCloseEvent& WXUNUSED(event)); // actions executed when the frame is closed

        void OnDblClickItem(wxListEvent& event); // actions executed when a listcontrol item is double-clicked

	private:
        void ReadReservations(); // reads DHCP reservations from registry
        void WriteReservations(); // writes DHCP reservations into registry

        Reservation GetReservationFromItem(long reservationItemIdx);
        void SetItemFromReservation(const Reservation& reservation, long reservationItemIdx);
        long AddReservationItem(const Reservation& reservation);

        long AddReservation(); // adds DHCP reservation
        void ModifyReservation(long currentReservationItemIdx); // modifies DHCP reservation
        void DeleteReservation(); // deletes DHCP reservation
        bool EditReservation(Reservation& reservation); // edits DHCP reservation

        bool IsModified(); // notices if DHCP reservations list has been modified

        ArqWx::ListCtrl* lstctrlReservations; // listcontrol to display DHCP reservations
        ArqWx::ListCtrl* lstctrlReservationsBackUp; // backup listcontrol created at start or when writing values into registry

        wxButton* btnRead;      // 'Read' button
        wxButton* btnWrite;     // 'Write' button

        wxButton* btnAdd;       // 'Add' button
        wxButton* btnModify;    // 'Modify' button
        wxButton* btnDelete;    // 'Delete' button

        wxRegKey* rkeyApp;      // application reg key


    DECLARE_EVENT_TABLE()
};

#endif // MYFRAME_H
